var _ = require('underscore');

var RestKit = {}
RestKit.globalOptions = {};
var Model = RestKit.Model = function (attributes) {
    var attrs = attributes || {};
    this.attributes = {};
    this.set(attrs);
    this.initialize.apply(this, arguments);
};

var verboseLogging = false;

// var getRequest = {
//     method: 'GET',
//     headers: RestKit.globalOptions.headers
// }

var postRequest = {
    method: 'POST',
    headers: RestKit.globalOptions.headers
}

var putRequest = {
    method: 'PUT',
    headers: RestKit.globalOptions.headers
}

var deleteRequest = {
    method: 'DELETE',
    headers: RestKit.globalOptions.headers
}

// Attach all inheritable methods to the Model prototype.
_.extend(Model.prototype, {
    idAttribute: '_id',
    initialize: function () {
    },
    toJSON: function (options) {
        return _.clone(this.attributes);
    },
    get: function (attr) {
        return this.attributes[attr];
    },
    has: function (attr) {
        return this.get(attr) != null;
    },
    matches: function (attrs) {
        return !!_.iteratee(attrs, this)(this.attributes);
    },
    set: function (field, value) {
        if (field == null) return this;
        var obj;
        if (typeof field === 'object') {
            obj = field;
        } else {
            (obj = {})[field] = value;
        }

        for (var field in obj) {
            if (obj[field] !== undefined) {
                this.attributes[field] = obj[field];
            }
        }

        return this;
    },

    // Remove an attribute from the model, firing `"change"`. `unset` is a noop
    // if the attribute doesn't exist.
    unset: function (attr) {
        delete this.attributes[attr];
    }
});

var extend = function (protoProps, staticProps) {
    var parent = this;
    var child;

    // The constructor function for the new subclass is either defined by you
    // (the "constructor" property in your `extend` definition), or defaulted
    // by us to simply call the parent constructor.
    if (protoProps && _.has(protoProps, 'constructor')) {
        child = protoProps.constructor;
    } else {
        child = function () {
            return parent.apply(this, arguments);
        };
    }

    // Add static properties to the constructor function, if supplied.
    _.extend(child, parent, staticProps);

    // Set the prototype chain to inherit from `parent`, without calling
    // `parent`'s constructor function and add the prototype properties.
    child.prototype = _.create(parent.prototype, protoProps);
    child.prototype.constructor = child;

    // Set a convenience property in case the parent's prototype is needed
    // later.
    child.__super__ = parent.prototype;

    return child;
};

RestKit.Model.prototype.post = function (url, body, callback) {
    return _post(url, body, callback);
}

RestKit.Model.prototype.sendMail = function (url, body) {
    return sendEmail(url, body);
}

RestKit.Model.prototype.query = function (fields) {
    return _query(this.urlRoot, fields, undefined, undefined, undefined, undefined);
}

RestKit.Model.prototype.query = function (fields, where) {
    return _query(this.urlRoot, fields, where, undefined, undefined, undefined);
}

RestKit.Model.prototype.query = function (fields, where, pageSize, callback) {
    _query(this.urlRoot, fields, where, pageSize, undefined, undefined, callback);
}

RestKit.Model.prototype.query = function (fields, where, pageSize, sortBy, offset, callback) {
    _query(this.urlRoot, fields, where, pageSize, sortBy, offset, callback);
}

var _query = function (url, fields, where, pageSize, sortBy, offset) {

    if (fields || where || pageSize || sortBy || offset) {
        url += "?";
    }
    if (fields !== undefined) {
        url += "props=" + fields + "&";
    }
    if (where !== undefined) {
        url += "where=" + where + "&";
    }
    if (pageSize !== undefined) {
        url += "pageSize=" + pageSize + "&";
    }
    if (sortBy !== undefined) {
        url += "sortBy=" + sortBy + "&";
    }
    if (offset !== undefined) {
        url += "offset=" + offset + "&";
    }
    url = url.substring(0, url.length - 1);
    console.log("fetching " + url);
    var getRequest = {
        method: 'GET',
        headers: RestKit.globalOptions.headers
    }
    return fetch(url, getRequest).then((response) => {
        return response.json()
    }).then((value) => {
        console.log(value);
        if (value.code !== undefined && value.code != 200) {
            return Promise.reject(value.code);
        }
        else {
            return value
        }
    }).catch((error) => {
        console.log(error);
        throw error;
    });
}

var _post = function (url, body) {
    var config = postRequest;
    if (body != null) {
        config.body = body;
    }
    return fetch(url, config)
        .then((response) => {
            console.log(response);
            if (response.status !== undefined && response.status != 200) {
                return Promise.reject(response.status);
            }
            else {
                return response.json()
            }
        })
        .catch((error) => {
            if (error == 3087) {
                throw "Email not yet confirmed";
            }
            else {
                throw error;
            }
        });
}

var _delete = function (url, body) {

    return fetch(url, deleteRequest)
        .then((jsonResponse) => {
            // here we have the error code (if any) in jsonResponse.code
            console.log("response is", jsonResponse);
            if (jsonResponse.code !== undefined && jsonResponse.code != 200) {
                console.log("rejecting promise with code", jsonResponse.code);
                var err = {errorCode: jsonResponse.code}
                return err
            }
            else {
                console.log("returning", jsonResponse);
                return jsonResponse
            }
        }).catch((e) => {
            console.log("catch");
            throw e;
        });

}

var _update = function (url, body) {
    return fetch(url, {
        method: 'PUT',
        headers: RestKit.globalOptions.headers,
        body: body
    }).then((response) => {
        console.log(response);
        if (response.status !== undefined && response.status == 404) {
            return Promise.reject('oh, no!');
        }
        else {
            return response.json()
        }
    }).then((jsonResponse) => {
        console.log(jsonResponse);
        if (jsonResponse.code !== undefined && jsonResponse.code != 200) {
            return Promise.reject(jsonResponse.code);
        }
        else {
            return jsonResponse
        }
    }).catch((error) => {
        console.log("catch");
        if (error == 3087) {
            throw "Email not yet confirmed";
        }
        else {
            throw error;
        }
    });
}


// Set up inheritance for the model, collection, router, view and history.
Model.extend = extend;

RestKit.Model = Model;

module.exports = RestKit;